# -*- coding: utf-8 -*-
import numpy as np
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
import matplotlib.colors as mpc
import mpmath as mp
#from matplotlib.patches import Rectangle

mp.mp.prec = 208

alphaD_vec = [mp.mpmathify(x) for x in [1.0E+2, 1.0E+5, 1.0E+8]]
KD_vec =     [mp.mpmathify(x) for x in [1.0E-1, 1.0E-4, 1.0E-7]]

nu = 450
ten = mp.mpmathify('10.0')
u_vec = [ten**x for x in mp.linspace(-25,6,nu)] # uniformly logspaced

Wu_vec =   list(map(mp.e1, u_vec)) # W(u)
invu_vec = list(map(lambda u:1/u, u_vec)) # 1/u

for psiD_src,pD_src,base in zip(['-2'],['2'],['neg-pos']):
    print(">"*50,"\n"+base,"\n" + ">"*50)
    
    psiD_grad_source = mp.mpmathify(psiD_src)  # electrical
    pD_grad_source   = mp.mpmathify(pD_src)  # hydrological

    fh = open('expint.csv','w')
    fh.write('#alphaD,KD,Delta,Lambda_1,Lambda_2,Gpp,Gp,Q1,Q2\n')

    lw = 1.5 # line width

    fig = plt.figure(1,figsize=(5,4))  # big delta1/delta2 fig
    ax = fig.add_subplot(111)
    ax.loglog(invu_vec, Wu_vec, 'b-', lw=lw, label='W$(u)$') # 1/u vs. W(u) :: original unshifted type curve

    figlin = plt.figure(8,figsize=(4.5,3.5))  # semilog-scale plot
    axlin = figlin.add_subplot(111)
    axlin.semilogx(invu_vec, Wu_vec, 'b-', lw=lw, label='W$(u)$') # 1/u vs. W(u) :: original unshifted type curve

    fig2 = plt.figure(2,figsize=(5,4)) # zoomed in delta1/delta2 fig
    axz = fig2.add_subplot(111)
    axz.loglog(invu_vec, Wu_vec, 'b-', lw=lw, label='W$(u)$') # 1/u vs. W(u) 

    fig3 = plt.figure(3,figsize=(7,3.5)) # recombined figure
    axp = fig3.add_subplot(121)
    axpsi = fig3.add_subplot(122)

    fig3lin = plt.figure(9,figsize=(4,3.5)) # recombined semilog-scale figure
    #axplin = fig3lin.add_subplot(211)
    axpsilin = fig3lin.add_subplot(111)

    lts = ['-','--',':','-.']
    black_colors = ['black','dimgray','gray']
    red_colors = ['maroon','red','orangered']

    print (f' psiD source: {repr(psiD_grad_source)}, pD source: {repr(pD_grad_source)}')
    print ('i j    alphaD        KD        Delta     Lambda_1    Lambda_2      Gp          Gpp          Q1            Q2')

    for ii,alphaD in enumerate(alphaD_vec):
        # alphaD -> line type
        for jj,KD in enumerate(KD_vec):
            # KD -> line color
            
            Delta = mp.sqrt(1 + alphaD * (4*KD - 2 + alphaD))

            Lambda_2 = (1 + alphaD + Delta) / 2
            Lambda_1 = alphaD*(1 - KD) / Lambda_2
            
            Gpp = (1 - (alphaD + Delta)) / 2
            Gp =  -KD * alphaD / Gpp

            # <x-shift> (argument to exponential integral)
            # log form: log(t/r**2) = log(1/4*Lambda) + log(1/u)
            # multiply form: u = r**2/(4*Lambda*t)
            x1_shift = 1/(4*Lambda_1)
            x2_shift = 1/(4*Lambda_2)

            Q1 = -(Gpp*pD_grad_source  + KD*psiD_grad_source)/Delta
            Q2 =  (Gp*pD_grad_source + KD*psiD_grad_source)/Delta

            print ('%i %i %.5E %.5E %.5E %.5E %.5E %.5E %.5E %.5E %.5E' %
                   (ii,jj,alphaD,KD,Delta,Lambda_1,Lambda_2,Gp,Gpp,Q1,Q2))

            # <y-shift> 
            # log form: log(s) = log(Q/4*pi*Lambda) + log(W(u))
            # multiply form: s = Q/(4*pi*Lambda)*W(u)
            y1_shift = Q1/(4*mp.pi*Lambda_1)
            y2_shift = Q2/(4*mp.pi*Lambda_2)

            fh.write('{},{},{},{},{},{},{},{},{}\n'\
                     .format(alphaD,KD,Delta,
                             Lambda_1,Lambda_2,
                             Gpp,Gp,Q1,Q2))
            
            # compute new y and x (for plotting individual terms)
            delta1  = np.array(list(map(lambda y: y*y1_shift, Wu_vec))) # new y-axis
            tovrsq1 = np.array(list(map(lambda x: x*x1_shift, invu_vec))) # new x-axis

            delta2 =  np.array(list(map(lambda y: y*y2_shift, Wu_vec)))
            tovrsq2 =          list(map(lambda x: x*x2_shift, invu_vec)) # leave as a list (for map)

            # version for plotting sum (delta1 at same points delta2 was computed at)
            invx2 =            list(map(lambda x: mp.e1(1/x),tovrsq2))
            delta1a = np.array(list(map(lambda y: y*y1_shift, invx2)))
            tovrsq2 = np.array(tovrsq2)
            
            # main figure
            # only the solid line type and darkest color variant gets a label
            if ii == 0 and jj == 0:
                label1 = '$\\delta_1$'
                label2 = '$\\delta_2$'
            else:
                label1 = None
                label2 = None
                
            ax.loglog(tovrsq1, np.abs(delta1), lw=lw, ls=lts[ii], color=red_colors[jj], label=label1)
            ax.loglog(tovrsq2, np.abs(delta2), lw=lw, ls=lts[ii], color=black_colors[jj], label=label2)
            axlin.semilogx(tovrsq1, delta1, lw=lw, ls=lts[ii], color=red_colors[jj], label=label1)
            axlin.semilogx(tovrsq2, delta2, lw=lw, ls=lts[ii], color=black_colors[jj], label=label2)

            # zoomed-in region
            axz.loglog(tovrsq1, np.abs(delta1), lw=lw, ls=lts[ii], color=red_colors[jj],label=label1)
            axz.loglog(tovrsq2, np.abs(delta2), lw=lw, ls=lts[ii], color=black_colors[jj],label=label2)

            psiD = -(Gp*delta1a + Gpp*delta2)/KD
            axpsi.loglog(tovrsq2, np.abs(psiD), lw=lw, ls=lts[ii], color=red_colors[jj])
            axpsilin.semilogx(tovrsq2, psiD, lw=lw, ls=lts[ii], color=red_colors[jj])
            
            pD = (delta1a + delta2)
            axp.loglog(tovrsq2, pD, lw=lw, ls=lts[ii], color=black_colors[jj])
                        
            #axplin.semilogx(tovrsq2, pD, lw=lw, ls=lts[ii], color=black_colors[jj])
            
    fh.close()

    ylz = [1.0E-2,2.0]
    xlz = [1.0E-1,1.0E+1]

    yl =    [1.0E-30,1.0E+2]
    ylpsi = [1.0E-15,1.0E+2]
    xl =    [1.0E-13,1.0E+3]

    xticks = [1.0E-12,1.0E-9,1.0E-6,1.0E-3,1.0,1.0E+3]
    
    ax.legend(loc=0)
    ax.set_xlabel('$t/r^2$')
    ax.set_ylabel('$|\\delta_i|$')
    ax.set_ylim(yl)
    ax.set_xlim(xl)
    ax.annotate("$\\alpha_D=10^{2}$",xy=(1.0E-3,1.0E-9),xytext=(1.0E-6,1.0E-6),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    ax.annotate("$\\alpha_D=10^{5}$",xy=(1.0E-6,1.0E-18),xytext=(1.0E-9,1.0E-14),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    ax.annotate("$\\alpha_D=10^{8}$",xy=(1.0E-9,1.0E-26),xytext=(1.0E-11,1.0E-22),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    ax.text(10,1.0E-6,"$K_D=10^{-1}$",ha="center",va="center",size=10,color=black_colors[0])
    ax.text(10,1.0E-9,"$K_D=10^{-4}$",ha="center",va="center",size=10,color=black_colors[1])
    ax.text(10,1.0E-12,"$K_D=10^{-7}$",ha="center",va="center",size=10,color=black_colors[2])
    rect = plt.Rectangle((xlz[0],ylz[0]),xlz[1]-xlz[0],ylz[1]-ylz[0],facecolor='gray',alpha=0.5)
    ax.add_patch(rect)
    fig.tight_layout()
    ax.set_xticks(xticks)
    ax.text(1.0E-9, 0.1, "a)",size=14)
    fig.savefig('expint_all_curves-'+base+'.png',dpi=300)
    plt.close(1)

    axlin.legend(loc=0)
    axlin.set_xlabel('$t/r^2$')
    axlin.set_ylabel('$\\delta_i$')
    figlin.tight_layout()
    figlin.savefig('expint_all_curves_semilog-'+base+'.png',dpi=300)
    plt.close(8)

    #axz.legend(loc=0)
    axz.set_xlabel('$t/r^2$')
    axz.set_ylabel('$|\\delta_i|$')
    axz.text(0.12,1.2,"b)",size=14)
    axz.text(5,0.6,"$K_D=10^{-1}$",ha="center",va="center",size=10,color=red_colors[0],rotation=15)
    axz.text(5,0.25,"$K_D \\leq 10^{-4}$",ha="center",va="center",size=10,color=red_colors[1],rotation=15)
    axz.set_ylim(ylz)
    axz.set_xlim(xlz)
    fig2.tight_layout()
    fig2.savefig('expint_all_curves_zoomed-'+base+'.png',dpi=300)
    plt.close(2)

    xlz2 = [0.01,1000.0]
    ylz2 = [0.001,2.0]

    axp.set_xlabel('$t/r^2$')
    axpsi.set_xlabel('$t/r^2$')
    axp.set_yticks(  [1.0E-30,1.0E-24,1.0E-18,1.0E-12,1.0E-6,1.0])
    axpsi.set_yticks([1.0E-30,1.0E-24,1.0E-18,1.0E-12,1.0E-6,1.0])
    
    axp.set_xticks(xticks)
    axpsi.set_xticks(xticks)
    axp.text(1.0E-12,0.1,"a)",size=14)
    axpsi.text(1.0E-12,0.1,"b)",size=14)
    axp.set_ylabel('$p$')
    axpsi.set_ylabel('$|\\psi|$')
    axp.annotate("$\\alpha_D=10^{2}$",xy=(1.0E-3,1.0E-9),xytext=(1.0E-6,1.0E-6),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    axp.annotate("$\\alpha_D=10^{5}$",xy=(1.0E-6,1.0E-18),xytext=(1.0E-9,1.0E-14),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    axp.annotate("$\\alpha_D=10^{8}$",xy=(1.0E-9,1.0E-26),xytext=(1.0E-11,1.0E-22),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    axp.text(0.1,1.0E-16,"$K_D=10^{-1}$",ha="left",va="center",color=black_colors[0],size=8)
    axp.text(0.1,1.0E-19,"$K_D=10^{-4}$",ha="left",va="center",color=black_colors[1],size=8)
    axp.text(0.1,1.0E-22,"$K_D=10^{-7}$",ha="left",va="center",color=black_colors[2],size=8)
    axp.set_ylim(yl)
    axp.set_xlim(xl)
    axpsi.set_ylim(yl)
    axpsi.set_xlim(xl)
    axpsi.annotate("$\\alpha_D=10^{2}$",xy=(1.0E-3,1.0E-7),xytext=(1.0E-7,1.0E-3),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    axpsi.annotate("$\\alpha_D=10^{5}$",xy=(1.0E-6,1.0E-12),xytext=(1.0E-10,1.0E-8),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    axpsi.annotate("$\\alpha_D=10^{8}$",xy=(1.0E-9,1.0E-18),xytext=(1.0E-12,1.0E-14),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=10)
    fig3.tight_layout()
    fig3.savefig('expint_all_curves_recombined-'+base+'.png',dpi=300)

    axpsilin.set_xlim(xl)
    axpsilin.set_xticks(xticks)
    axpsilin.set_yscale('symlog',linthreshy=1.0E-18)
    axpsilin.axhline(0.0,color='gray',lw=0.25,zorder=-100)
    axpsilin.set_yticks([-1.0,-1.0E-4,-1.0E-8,-1.0E-12,-1.0E-16,0.0,1.0E-16,1.0E-12,1.0E-8,1.0E-4,1.0])
    anns = []
    anns.append(axpsilin.annotate("$\\alpha_D=10^{2}$",xy=(1.0E-3,1.0E-7),xytext=(1.0E-7,1.0E-3),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8))
    anns.append(axpsilin.annotate("$\\alpha_D=10^{5}$",xy=(1.0E-6,1.0E-12),xytext=(1.0E-10,1.0E-8),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8))
    anns.append(axpsilin.annotate("$\\alpha_D=10^{8}$",xy=(1.0E-9,1.0E-18),xytext=(1.0E-12,1.0E-14),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8))

    axpsilin.set_ylabel('$\\psi$')
    axpsilin.set_xlabel('$t/r^2$')
    axpsilin.text(1.0E-12,-1.0E-2,"a)",size=14)
    fig3lin.tight_layout()
    fig3lin.savefig('expint_all_curves_recombined_symlog-'+base+'.png',dpi=300)

    for ann in anns:
        ann.remove()
    
    axpsilin.set_yscale('linear')
    axpsilin.text(1.0E-12,-4.5E-5,"b)",size=14)
    axpsilin.set_ylim([-0.55E-4,0.55E-4])
    axpsilin.set_yticks([-0.5E-4,-0.25E-4,0,0.25E-4,0.5E-4])
    axpsilin.set_yticklabels(["$-5 \\times 10^{-5}$","$-2.5 \\times 10^{-5}$","0","$2.5 \\times 10^{-5}$","$5 \\times 10^{-5}$"])
    axpsilin.annotate("$\\alpha_D=10^{2}$",xy=(1.0E-2,1.0E-5),xytext=(1.0E-7,1.5E-5),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8)
    axpsilin.annotate("$\\alpha_D=10^{5}$",xy=(1.0E-2,0),xytext=(1.0E-5,-1.0E-5),ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8)
    axpsilin.annotate("$K_D=10^{-1}$",xy=(0.8E-1,4.0E-5),xytext=(1.0E-7,3.0E-5),color=red_colors[0],ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8)
    axpsilin.annotate("$K_D \\leq 10^{-4}$",xy=(1.0E-1,5.0E-5),xytext=(1.0E-7,5.0E-5),color=red_colors[1],ha="left",va="center",arrowprops={'arrowstyle':'->'},size=8)
    #fig3lin.tight_layout()
    fig3lin.savefig('expint_all_curves_recombined_semilog-'+base+'.png',dpi=300)
    plt.close(9)

    axp.set_ylim(ylz2)
    axp.set_xlim(xlz2)
    axpsi.set_ylim(ylz2)
    axpsi.set_xlim(xlz2)
    axp.set_yticks(  [0.001,0.01,0.1,1.0])
    axpsi.set_yticks([0.001,0.01,0.1,1.0])
    axp.set_xticks(  [0.01,0.1,1,10.0,1.0E+2,1.0E+3])
    axpsi.set_xticks([0.01,0.1,1,10.0,1.0E+2,1.0E+3])
    fig3.tight_layout()
    fig3.savefig('expint_all_curves_recombined_zoomed-'+base+'.png',dpi=300)

    plt.close(3)



        
